@@ -5,8 +5,11 @@ leuvenmapmatching.matcher.base
 
 Base Matcher and Matching classes.
 
+This a generic base class to be used by matchers. This class itself
+does not implement a working matcher.
+
 :author: Wannes Meert
-:copyright: Copyright 2015-2018 DTAI, KU Leuven and Sirris.
+:copyright: Copyright 2015-2021 DTAI, KU Leuven and Sirris.
 :license: Apache License, Version 2.0, see LICENSE for details.
 """
 from __future__ import print_function
@@ -256,7 +259,7 @@ class BaseMatching(object):
                                 self.dist_obs, ",".join([str(prev.label) for prev in self.prev]))
 
     def __repr__(self):
-        return self.label
+        return "Matching<"+str(self.label)+">"
 
     @staticmethod
     def repr_header(label_width=None, stop=""):
@@ -336,6 +339,12 @@ class LatticeColumn:
     def __len__(self):
         return len(self.o)
 
+    def set_delayed(self, delayed):
+        """Update all delayed values."""
+        for c in self.o:
+            for m in c.values():
+                m.delayed = delayed
+
     def dict(self, obs_ne=None):
         if obs_ne is None:
             raise AttributeError('obs_ne should be value')
@@ -425,6 +434,9 @@ class BaseMatcher:
                  non_emitting_length_factor=0.75, **kwargs):
         """Initialize a matcher for map matching.
 
+        This a generic base class to be used by matchers. This class itself
+        does not implement a working matcher.
+
         Distances are in meters when using latitude-longitude.
 
         :param map_con: Map object to connect to map database
@@ -454,7 +466,7 @@ class BaseMatcher:
         - :meth:`logprob_obs`
 
         """
-        self.map = map_con
+        self.map = map_con  # type: BaseMap
         if max_dist:
             self.max_dist = max_dist
         else:
@@ -476,13 +488,14 @@ class BaseMatcher:
 
         self.path = None
         self.lattice = None  # type: Optional[dict[int,LatticeColumn]]
-        self.lattice_best = None
-        self.node_path = None
+        # Best path through lattice:
+        self.lattice_best = None  # type: Optional[list[BaseMatching]]
+        self.node_path = None  # type: Optional[list[str]]
         self.matching = matching
-        self.non_emitting_states = non_emitting_states
+        self.non_emitting_states = non_emitting_states  # type: bool
         self.non_emitting_states_maxnb = 100
-        self.max_lattice_width = max_lattice_width
-        self.only_edges = only_edges
+        self.max_lattice_width = max_lattice_width  # type: Optional[int]
+        self.only_edges = only_edges  # type: bool
         self.expand_now = 0  # all m.delayed <= expand_upto will be expanded
 
         # Penalties
@@ -539,16 +552,31 @@ class BaseMatcher:
         :param unique: Only retain unique nodes in the sequence (avoid repetitions)
         :param tqdm: Use a tqdm progress reporter (default is None)
         :param expand: Expand the current lattice (delayed matches)
-        :return: Tuple of (List of state keys, index of last observation that was matched)
+        :return: Tuple of (List of BaseMatching, index of last observation that was matched)
         """
         if __debug__:
             logger.debug("Start matching path of length {}".format(len(path)))
 
         # Initialisation
         if expand:
-            if self.path != path:
-                raise Exception(f'Cannot expand for a new path, should be the same path.')
             self.expand_now += 1
+            if self.path != path:
+                is_path_extended = True
+                if len(path) > len(self.path):
+                    for pi, spi in zip(path, self.path):
+                        if pi != spi:
+                            is_path_extended = False
+                            break
+                else:
+                    is_path_extended = False
+                if is_path_extended:
+                    self.lattice[len(self.path) - 1].set_delayed(self.expand_now)
+                    for obs_idx in range(len(self.path), len(path)):
+                        if obs_idx not in self.lattice:
+                            self.lattice[obs_idx] = LatticeColumn(obs_idx)
+                    self.path = path
+                else:
+                    raise Exception(f'Cannot expand for a new path, should be the same path (or an extension).')
         else:
             self.path = path
             self.expand_now = 0
@@ -1509,3 +1537,35 @@ class BaseMatcher:
         if self.node_path is None or len(self.node_path) == 0:
             return []
         return self.node_path_to_only_nodes(self.node_path)
+
+    def path_pred_distance(self):
+        """Total distance of the matched path."""
+        if self.lattice_best is None:
+            return None
+        if len(self.lattice_best) == 1:
+            return 0
+        dist = 0
+        m_prev = self.lattice_best[0]
+        for idx, m in enumerate(self.lattice_best[1:]):
+            if m_prev.edge_m.label != m.edge_m.label and m_prev.edge_m.l2 == m.edge_m.l1:
+                # Go over the connection between two edges to compute the distance
+                cdist = self.map.distance(m_prev.edge_m.pi, m_prev.edge_m.p2)
+                cdist += self.map.distance(m_prev.edge_m.p2, m.edge_m.pi)
+            else:
+                cdist = self.map.distance(m_prev.edge_m.pi, m.edge_m.pi)
+            dist += cdist
+            m_prev = m
+        return dist
+
+    def path_distance(self):
+        """Total distance of the observations."""
+        if self.lattice_best is None:
+            return None
+        if len(self.lattice_best) == 1:
+            return 0
+        dist = 0
+        m_prev = self.lattice_best[0]
+        for m in self.lattice_best[1:]:
+            dist += self.map.distance(m_prev.edge_o.pi, m.edge_o.pi)
+            m_prev = m
+        return dist
