@@ -358,6 +358,48 @@ def test_path3_dist():
     assert path_pred == path_sol, "Nodes not equal:\n{}\n{}".format(path_pred, path_sol)
 
 
+def test_path4_dist_inc():
+    map_con = InMemMap("mymap", graph={
+        "A": ((1, 1), ["B", "C", "X"]),
+        "B": ((1, 3), ["A", "C", "D", "K"]),
+        "C": ((2, 2), ["A", "B", "D", "E", "X", "Y"]),
+        "D": ((2, 4), ["B", "C", "D", "E", "K", "L"]),
+        "E": ((3, 3), ["C", "D", "F", "Y"]),
+        "F": ((3, 5), ["D", "E", "L"]),
+        "X": ((2, 0), ["A", "C", "Y"]),
+        "Y": ((3, 1), ["X", "C", "E"]),
+        "K": ((1, 5), ["B", "D", "L"]),
+        "L": ((2, 6), ["K", "D", "F"])
+    }, use_latlon=False)
+
+    path = [(0.8, 0.7), (0.9, 0.7), (1.1, 1.0), (1.2, 1.5), (1.2, 1.6), (1.1, 2.0),
+            (1.1, 2.3), (1.3, 2.9), (1.2, 3.1), (1.5, 3.2), (1.8, 3.5), (2.0, 3.7),
+            (2.3, 3.5), (2.4, 3.2), (2.6, 3.1), (2.9, 3.1), (3.0, 3.2),
+            (3.1, 3.8), (3.0, 4.0), (3.1, 4.3), (3.1, 4.6), (3.0, 4.9)]
+
+    matcher = DistanceMatcher(map_con, max_dist=2, obs_noise=1, min_prob_norm=0.5)
+    matcher.match(path[:5])
+
+    # states, _ = matcher.match_incremental(path[:5])
+    # states, _ = matcher.match_incremental(path[5:], backtrace_len=-1)
+    nodes = matcher.path_pred_onlynodes
+    print(nodes)
+
+    if directory:
+        from leuvenmapmatching import visualization as mmviz
+        mmviz.plot_map(map_con, matcher=matcher,
+                       show_labels=True, show_matching=True, show_graph=True,
+                       filename=str(directory / "test_path4_dist_inc_1.png"))
+
+    matcher.match(path, expand=True)
+
+    if directory:
+        from leuvenmapmatching import visualization as mmviz
+        mmviz.plot_map(map_con, matcher=matcher,
+                       show_labels=True, show_matching=True, show_graph=True,
+                       filename=str(directory / "test_path4_dist_inc_2.png"))
+
+
 if __name__ == "__main__":
     logger.setLevel(logging.DEBUG)
     logger.addHandler(logging.StreamHandler(sys.stdout))
@@ -367,9 +409,10 @@ if __name__ == "__main__":
     # test_path1_dist()
     # test_path2()
     # test_path2_inc()
-    test_path2_dist()
+    # test_path2_dist()
     # test_path_outlier()
     # test_path_outlier2()
     # test_path_outlier_dist()
     # test_path3()
     # test_path3_dist()
+    test_path4_dist_inc()
